'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var slate = require('slate');
var slateHistory = require('slate-history');
var slateReact = require('slate-react');
var ulid = require('ulid');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

function debounce(func, waitMilliseconds, options) {
    var _a, _b;
    if (waitMilliseconds === void 0) { waitMilliseconds = 50; }
    if (options === void 0) { options = {}; }
    var timeoutId;
    var isImmediate = (_a = options.isImmediate) !== null && _a !== void 0 ? _a : false;
    var callback = (_b = options.callback) !== null && _b !== void 0 ? _b : false;
    var maxWait = options.maxWait;
    var lastInvokeTime = Date.now();
    var promises = [];
    function nextInvokeTimeout() {
        if (maxWait !== undefined) {
            var timeSinceLastInvocation = Date.now() - lastInvokeTime;
            if (timeSinceLastInvocation + waitMilliseconds >= maxWait) {
                return maxWait - timeSinceLastInvocation;
            }
        }
        return waitMilliseconds;
    }
    var debouncedFunction = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var context = this;
        return new Promise(function (resolve, reject) {
            var invokeFunction = function () {
                timeoutId = undefined;
                lastInvokeTime = Date.now();
                if (!isImmediate) {
                    var result_1 = func.apply(context, args);
                    callback && callback(result_1);
                    promises.forEach(function (_a) {
                        var resolve = _a.resolve;
                        return resolve(result_1);
                    });
                    promises = [];
                }
            };
            var shouldCallNow = isImmediate && timeoutId === undefined;
            if (timeoutId !== undefined) {
                clearTimeout(timeoutId);
            }
            timeoutId = setTimeout(invokeFunction, nextInvokeTimeout());
            if (shouldCallNow) {
                var result = func.apply(context, args);
                callback && callback(result);
                return resolve(result);
            }
            promises.push({ resolve: resolve, reject: reject });
        });
    };
    debouncedFunction.cancel = function (reason) {
        if (timeoutId !== undefined) {
            clearTimeout(timeoutId);
        }
        promises.forEach(function (_a) {
            var reject = _a.reject;
            return reject(reason);
        });
        promises = [];
    };
    return debouncedFunction;
}

/**
 * Returns a random integer between min (inclusive) and max (inclusive).
 * The value is no lower than min (or the next integer greater than min
 * if min isn't an integer) and no greater than max (or the next integer
 * lower than max if max isn't an integer).
 * Using Math.round() will give you a non-uniform distribution!
 */
function getRandomInt(min, max) {
    min = Math.ceil(min);
    max = Math.floor(max);
    return Math.floor(Math.random() * (max - min + 1)) + min;
}

function throttle(func, limit) {
    var inThrottle;
    var lastResult;
    return function () {
        var args = arguments;
        var context = this;
        if (!inThrottle) {
            inThrottle = true;
            setTimeout(function () { return (inThrottle = false); }, limit);
            lastResult = func.apply(context, args);
        }
        return lastResult;
    };
}

/**
 * Responsible for fetching completions from a model and adding them to the input.
 * Each `<SmartInput />` component rendered to the DOM will have its own instance of this class as state.
 */
var SmartInputFetcher = /** @class */ (function () {
    function SmartInputFetcher(apiUrl, addCompletion, modelId, mockCompletion) {
        var _this = this;
        this.apiUrl = apiUrl;
        this.addCompletion = addCompletion;
        this.modelId = modelId;
        this.mockCompletion = mockCompletion;
        /**
         * sequenceId is used to ensure that we don't add completions from a previous fetch,
         * we increment it each time we request to fetch a new completion because the user input has changed
         */
        this.sequenceId = 0;
        /**
         * sessionId is used to track this particular instantiation of the `<SmartInput />` component
         */
        this.sessionId = ulid.ulid();
        /**
         * completionCache is used to cache the completions that we've already fetched from the model
         */
        this.completionCache = {};
        /**
         * alreadySeen tracks which completions we've marked as seen (reduce chattiness)
         */
        this.alreadySeen = new Set();
        /**
         * alreadyAccepted tracks which completions we've marked as acceoted (reduce chattiness)
         */
        this.alreadyAccepted = new Set();
        /**
         * currentCompletion tracks the latest text that was sent to the model for fetching completions, and the
         * completion that was returned (if any ~ we may still be waiting for it)
         */
        this.currentCompletion = null;
        this.fetchCompletion = debounce(function (text, context) { return __awaiter(_this, void 0, void 0, function () {
            var seqId, currExpectedText, cachedVal, completion_1, res, data;
            var _this = this;
            var _a, _b;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        if (text.length === 0) {
                            // only get completions for non-empty text
                            return [2 /*return*/, null];
                        }
                        seqId = ++this.sequenceId;
                        // if a completion was already added, and the user is typing the same characters as the current completion, don't fetch
                        if ((_a = this.currentCompletion) === null || _a === void 0 ? void 0 : _a.completion) {
                            currExpectedText = this.currentCompletion.text + this.currentCompletion.completion;
                            if (currExpectedText.startsWith(text)) {
                                return [2 /*return*/, null];
                            }
                        }
                        // update the current completion
                        this.currentCompletion = { text: text, sequenceId: seqId };
                        cachedVal = this.completionCache[text];
                        if (cachedVal) {
                            // don't fetch the same text twice
                            this.addCompletion(text, cachedVal);
                            this.currentCompletion.completion = cachedVal.completion;
                            return [2 /*return*/, cachedVal];
                        }
                        if (!this.mockCompletion) return [3 /*break*/, 3];
                        completion_1 = this.mockCompletion(text);
                        if (!completion_1) return [3 /*break*/, 2];
                        return [4 /*yield*/, new Promise(function (resolve) {
                                setTimeout(function () {
                                    var _a;
                                    var c = { input: text, sid: ulid.ulid(), completion: completion_1 };
                                    _this.completionCache[text] = c;
                                    if (((_a = _this.currentCompletion) === null || _a === void 0 ? void 0 : _a.sequenceId) === seqId) {
                                        _this.addCompletion(text, c);
                                        _this.currentCompletion.completion = completion_1;
                                    }
                                    resolve(c);
                                }, getRandomInt(200, 1000)); // simulate API latency
                            })];
                    case 1: return [2 /*return*/, _c.sent()];
                    case 2: return [2 /*return*/, { input: text, sid: ulid.ulid(), completion: null }];
                    case 3: return [4 /*yield*/, fetch("".concat(this.apiUrl, "/completions"), {
                            method: 'POST',
                            mode: 'cors',
                            headers: {
                                'Content-Type': 'application/json',
                                Authorization: "Bearer ".concat(this.modelId),
                            },
                            body: JSON.stringify({
                                model: this.modelId,
                                input: text,
                                session: this.sessionId,
                                context: context,
                            }),
                        })];
                    case 4:
                        res = _c.sent();
                        if (res.status !== 200) {
                            console.error("Couldn't fetch completions");
                            return [2 /*return*/, null];
                        }
                        return [4 /*yield*/, res.json()];
                    case 5:
                        data = (_c.sent());
                        this.completionCache[text] = data;
                        if (((_b = this.currentCompletion) === null || _b === void 0 ? void 0 : _b.sequenceId) === seqId) {
                            this.addCompletion(text, data);
                            this.currentCompletion.completion = data.completion;
                        }
                        return [2 /*return*/, data];
                }
            });
        }); }, 200);
        this.sendEvents = throttle(function (text, context) {
            fetch("".concat(_this.apiUrl, "/events"), {
                method: 'POST',
                mode: 'cors',
                headers: {
                    'Content-Type': 'application/json',
                    Authorization: "Bearer ".concat(_this.modelId),
                },
                body: JSON.stringify({
                    model: _this.modelId,
                    session: _this.sessionId,
                    input: text,
                    context: context,
                }),
            })
                .then(function (res) {
                if (res.status !== 200) {
                    console.error('Error sending events');
                }
            })
                .catch(function (e) { return console.error(e); });
        }, 1000);
    }
    SmartInputFetcher.prototype.updateCompletion = function (completionSid, arg) {
        var _this = this;
        if (this.mockCompletion) {
            return;
        }
        // reduce chattiness
        var shouldUpdate = (arg.seen && !this.alreadySeen.has(completionSid)) || (arg.accepted && !this.alreadyAccepted.has(completionSid));
        if (!shouldUpdate) {
            return;
        }
        fetch("".concat(this.apiUrl, "/completions"), {
            method: 'PUT',
            mode: 'cors',
            headers: {
                'Content-Type': 'application/json',
                Authorization: "Bearer ".concat(this.modelId),
            },
            body: JSON.stringify({
                completionSid: completionSid,
                seen: arg.seen,
                accepted: arg.accepted,
            }),
        })
            .then(function (res) {
            if (res.status !== 200) {
                console.error("Couldn't update completion");
            }
            if (arg.seen) {
                // only update the set if the update was successful
                _this.alreadySeen.add(completionSid);
            }
            if (arg.accepted) {
                // only update the set if the update was successful
                _this.alreadyAccepted.add(completionSid);
            }
        })
            .catch(function (e) { return console.error(e); });
    };
    return SmartInputFetcher;
}());

console.log('here yep yep');
/**
 * A text input that provides intelligent, tab-completable completions based on the text that has already been entered.
 * Due to the limitations of `<input>` and `<textarea>` elements, this component renders a `<div contenteditable="true">` instead.
 */
function SmartInput(props) {
    var editor = React__default["default"].useMemo(function () { return slateHistory.withHistory(slateReact.withReact(slate.createEditor())); }, []);
    var _a = React__default["default"].useState(null), completion = _a[0], setCompletion = _a[1];
    var fetcher = React__default["default"].useState(new SmartInputFetcher(props.devApiUrl || 'https://api.foundation-ui.com/smartinput', function (text, completion) {
        addCompletion(completion);
    }, props.modelId, props.modelId ? props.mockCompletion : function () { return ' -- sample completion --'; }))[0];
    var acceptCompletion = function () {
        EditorHelper.acceptCompletion(editor, fetcher, completion);
        setCompletion(null);
    };
    var addCompletion = function (completion) {
        if (props.value) {
            // don't add completion if the text is gone
            EditorHelper.addCompletion(editor, fetcher, completion);
            setCompletion(completion);
        }
    };
    var clearCompletion = function () {
        EditorHelper.removeCompletion(editor);
        setCompletion(null);
    };
    // for good measure, always overwrite the fetcher fields
    // fetcher.modelId = props.modelId
    fetcher.addCompletion = function (text, completion) {
        var currText = props.value;
        if (completion && (text + completion).startsWith(currText)) {
            addCompletion(completion);
        }
    };
    // fetcher.mockCompletion = props.mockCompletion
    var initialValue = [
        {
            type: 'text',
            children: [{ text: props.value, type: undefined }],
        },
    ];
    React.useEffect(function () {
        if (props.editorRef) {
            props.editorRef.current = __assign(__assign({}, slateReact.ReactEditor.toDOMNode(editor, editor)), { focus: function () {
                    setTimeout(function () {
                        slateReact.ReactEditor.focus(editor);
                        if (props.value) {
                            slate.Transforms.select(editor, slate.Editor.end(editor, []));
                        }
                        else {
                            slate.Transforms.select(editor, slate.Editor.start(editor, []));
                        }
                    }, 50);
                } });
        }
    });
    var renderElement = React__default["default"].useCallback(function (p) {
        switch (p.element.type) {
            case 'text':
                return props.renderText(__assign({}, p));
            case 'completion':
                return props.renderCompletion(__assign(__assign({}, p), { contentEditable: 'false', suppressContentEditableWarning: true }));
            default:
                return React__default["default"].createElement(slateReact.DefaultElement, __assign({}, p));
        }
    }, 
    // eslint-disable-next-line react-hooks/exhaustive-deps
    [props.renderText, props.renderCompletion]);
    var renderPlaceholder = React__default["default"].useCallback(function (p) {
        return props.renderPlaceholder(__assign(__assign(__assign({}, p), (p.attributes || {})), { style: __assign({ 
                // these style attributes usually come from slate, we gotta make sure to add them back or the cursor might go after the placeholder lol
                position: 'absolute', pointerEvents: 'none', width: '100%', maxWidth: '100%', userSelect: 'none', textDecoration: 'none' }, (p.style || {})) }));
    }, [props.renderPlaceholder]);
    return (React__default["default"].createElement(slateReact.Slate, { editor: editor, value: initialValue, onChange: function (value) {
            var _a;
            var text = EditorHelper.serialize(value.filter(function (n) { return n.type !== 'completion'; }));
            (_a = fetcher.fetchCompletion(text, props.context || {})) === null || _a === void 0 ? void 0 : _a.catch(function (e) { return console.error(e); });
            fetcher.sendEvents(text, props.context || {});
            props.onChange(text);
        } },
        React__default["default"].createElement(slateReact.Editable, { placeholder: props.placeholder, className: props.className, style: __assign(__assign({}, (props.style || {})), { whiteSpace: props.multiline ? undefined : 'pre' }), onPaste: function (event) {
                clearCompletion();
                event.preventDefault();
                var value = event.clipboardData.getData('Text');
                if (!props.multiline) {
                    editor.insertText(value.replaceAll('\n', ''));
                }
                else {
                    editor.insertText(value);
                }
            }, renderPlaceholder: props.renderPlaceholder ? renderPlaceholder : undefined, renderElement: renderElement, onBlur: function (e) {
                clearCompletion();
                fetcher.currentCompletion = null;
                if (props.onBlur) {
                    props.onBlur(e);
                }
            }, onFocus: function (e) {
                if (props.onFocus) {
                    props.onFocus(e);
                    if (editor.selection == null) {
                        // fix bug for chrome
                        if (props.value) {
                            slate.Transforms.select(editor, slate.Editor.end(editor, []));
                        }
                        else {
                            slate.Transforms.select(editor, slate.Editor.start(editor, []));
                        }
                    }
                }
            }, onKeyDown: function (event) {
                var _a, _b, _c;
                var hasCompletion = Boolean((_b = (_a = completion === null || completion === void 0 ? void 0 : completion.completion) === null || _a === void 0 ? void 0 : _a.trim()) === null || _b === void 0 ? void 0 : _b.length);
                if (event.key === 'Tab' && hasCompletion) {
                    event.preventDefault();
                    acceptCompletion();
                    fetcher.currentCompletion = null;
                }
                else if (event.key === 'ArrowRight' && hasCompletion) {
                    event.preventDefault();
                    acceptCompletion();
                    fetcher.currentCompletion = null;
                }
                else if (event.key === 'ArrowLeft' && hasCompletion) {
                    clearCompletion();
                    fetcher.currentCompletion = null;
                }
                else if ((event.key === 'ArrowDown' || event.key === 'ArrowUp') && hasCompletion) {
                    clearCompletion();
                }
                else if (event.key === 'Backspace') {
                    fetcher.currentCompletion = null;
                    clearCompletion();
                }
                else if (event.key === 'Enter' && !event.metaKey) {
                    event.preventDefault();
                    if (props.multiline) {
                        clearCompletion();
                        editor.insertText('\n');
                    }
                }
                else if (event.key !== 'Shift') {
                    if (completion &&
                        !event.metaKey &&
                        ((_c = completion.completion) === null || _c === void 0 ? void 0 : _c.startsWith(event.key)) &&
                        event.key.length === 1) {
                        var newCompletionText = completion.completion.substring(1);
                        // don't clear the completion if the user is just typing the completion character for character
                        EditorHelper.updateCompletion(editor, newCompletionText);
                        setCompletion(__assign(__assign({}, completion), { completion: newCompletionText }));
                    }
                    else {
                        fetcher.currentCompletion = null;
                        clearCompletion(); // TODO: this is a bit aggressive?
                    }
                }
                if (props.onKeyDown) {
                    props.onKeyDown(event);
                }
            } })));
}
var EditorHelper = {
    isCompletionActive: function (editor) {
        var completion = editor.children.find(function (n) { return n.type === 'completion'; });
        return !!completion;
    },
    isCursorAtEndOfDocument: function (editor) {
        var sel = editor.selection;
        if (!sel) {
            return false;
        }
        var node = sel.anchor.path[0];
        var offset = sel.anchor.offset;
        if (node !== editor.children.length - 1) {
            // not at the last node
            return false;
        }
        return offset === slate.Node.string(editor.children[node]).length;
    },
    acceptCompletion: function (editor, fetcher, completion) {
        var isActive = EditorHelper.isCompletionActive(editor);
        if (isActive && (completion === null || completion === void 0 ? void 0 : completion.completion)) {
            EditorHelper.removeCompletion(editor);
            editor.insertText(completion.completion);
            fetcher.updateCompletion(completion.sid, { accepted: true });
        }
    },
    addCompletion: function (editor, fetcher, completion) {
        var isActive = EditorHelper.isCompletionActive(editor);
        var isCursorEnd = EditorHelper.isCursorAtEndOfDocument(editor);
        if (!isActive && isCursorEnd && (completion === null || completion === void 0 ? void 0 : completion.completion)) {
            slate.Transforms.insertNodes(editor, {
                type: 'completion',
                children: [{ text: completion.completion }],
            }, {
                at: [editor.children.length],
            });
            fetcher.updateCompletion(completion.sid, { seen: true });
        }
    },
    removeCompletion: function (editor) {
        var isActive = EditorHelper.isCompletionActive(editor);
        if (isActive) {
            slate.Transforms.removeNodes(editor, {
                at: [editor.children.length - 1],
            });
        }
    },
    updateCompletion: function (editor, newCompletion) {
        var isActive = EditorHelper.isCompletionActive(editor);
        if (isActive && newCompletion.length > 0) {
            // could not find a way to update to set the text of the node, so we remove the first character only
            slate.Transforms.delete(editor, {
                at: {
                    anchor: { path: [editor.children.length - 1, 0], offset: 0 },
                    focus: { path: [editor.children.length - 1, 0], offset: 1 },
                },
            });
        }
    },
    serialize: function (nodes) {
        return nodes.map(function (n) { return slate.Node.string(n); }).join('\n');
    },
};

exports.SmartInput = SmartInput;
//# sourceMappingURL=bundle.cjs.map
